#include "BF_LCMV.h"
#include <../config.h>
#include <stdio.h>
#include <iostream>

namespace BF
{
    LCMV::LCMV(int _numFreq, int _lowerfreq, int _upperfreq, int _samplingRate, double _soundSpeed, bool _muAdp, double _mu, UtUpdateType _separationMatUpdateType) :
	BF(_numFreq, _lowerfreq, _upperfreq, _samplingRate, _soundSpeed, _separationMatUpdateType),
	muAdp(_muAdp),
	mu(_mu)
    {
    }

    LCMV::~LCMV()
    {
    }

    void
    LCMV::updateSources(const std::vector<SourceLocation> &sourcePositions)
    {
/*== source from adaptBF.m

  % [1] O. L. FrostIII, 'An Algorithm for Lineary Constrained Adaptive
  % array processing', Proc. of the IEEE, Vol. 60, No.8, 1972
  % The difference between this program and the paper are only two points: 
  % 1: This uses complex. Transpose is processed with complex conjugation.
  % 2: This uses row vectors for BF coefficient. (Paper uses column vectors)
      if isempty(Adata.F) % Initialization process
        Adata.F = zeros(NSrc, NMic, NFreq); % LCMV parameter 1 (F)
        Adata.P = zeros(NMic, NMic, NSrc, NFreq); % LCMV parameter 2 (P)
        for nf = 1:NFreq
          for ns = 1:NSrc % for each source
            Fi = 1; % Constrained target value
            C = Adata.H(:, ns, nf); % Constraned base vector
            Adata.F(ns, :, nf) = Fi*inv(C'*C)*C'; % = Fi*pinv(C) = WDS-BF
            Adata.P(:, :, ns, nf) = eye(NMic) - C*inv(C'*C)*C';
          end
        end
        if isempty(G), G = Adata.F; end
        return
        
      else % update process
        for nf = 1:NFreq
          for ns = 1:NSrc % for each source
              g = G(ns, :, nf);
              F = Adata.F(ns, :, nf);
              P = Adata.P(:, :, ns, nf);
              myu = Adata.myu;
              y = Adata.y(ns, nf);
              x = Adata.x(:, nf);
              gn = (g - myu*y*x') * P + F; % Update Eq. (22) in [1]
              G(ns, :, nf) = gn;
          end
        end
        return
      end % For if initialization
      
*/

	int numSrc = sourcePositions.size();

	/*== 直近フレームでの音源と、このフレームでの音源との対応をとる ==*/
#if 0 // 2014/10/14 分離行列引継ぎのために書き換え。とりあえず前のソースを残している
	std::vector<int> lastSourceIndices; // i 番めの要素は、このフレームでの i 番目の音源（配列の i 番目の要素という意味）が、直近フレームでの何番目の音源（同）に相当するかを表す。
	for(int i = 0; i < numSrc; i++){
	    const SourceLocation &sourcePosition = sourcePositions[i];

	    /* 直近フレームの音源から、同じ ID を持つものを探す */
	    int j;
	    for(j = 0; j < this->sourcePositions.size(); j++){
		if(sourcePosition.sourceID == this->sourcePositions[j].sourceID){
		    break;
		}
	    }
	    
	    if(j < this->sourcePositions.size()){
		/* 見つかった */
		lastSourceIndices.push_back(j);
	    }else{
		/* 見つからなかった */
		lastSourceIndices.push_back(-1);
	    }
	}
#endif

	/*== 直近フレーム・分離行列データベースの音源と、このフレームでの音源との対応をとる ==*/
	std::vector<int> sourceIndices;
	std::vector<int> sourceIndicestypes;
	getSourceIndex(sourcePositions, sourceIndicestypes, sourceIndices);	

	/*== 伝達関数を再計算する ==*/
	std::vector<Matrix> newHs;
	this->BF::getTransformFunctions(sourcePositions, newHs);

	std::vector<Matrix> newGs;
	std::vector<Matrix> newFs;
	std::vector<Matrix> newPs;
//	for(int i = lowerfreq; i <= upperfreq ; i++){
	for(int i = 0; i < this->numFreq; i++){
	    const Matrix &H = newHs[i];
	    int numMic = H.getRowCount();

		Matrix G(numSrc, numMic);
		Matrix F(numSrc, numMic);
	    
		// 対象周波数のみ分析
	    if (i == 0 || i == this->numFreq-1 || i < lowerfreq || i > upperfreq) {
			F = Matrix::O(numSrc, numMic);
		    for(int j = 0; j < numSrc; j++){
				Matrix P = Matrix::O(numMic, numMic);
				newPs.push_back(P);
			}
			G = Matrix::O(numSrc, numMic);
		}
		else {
		    for(int j = 0; j < numSrc; j++){
				Matrix P;

				/* 入力の j 番目の音源と同じ ID の音源が、直近フレームにあるか、データベースにあるか、見当たらないか？ */
				const int sourceIndex = sourceIndices[j];
				const int type = sourceIndicestypes[j];
				/*if (type == 0) { // 直近フレームにある
					// 直近フレームのデータからの引き継ぎ
					F.setRow(j, this->F[i].getRow(sourceIndex));
					P = this->P[i * this->sourcePositions.size() + sourceIndex];
					G.setRow(j, this->separationMatrices[i].getRow(sourceIndex));
				}
				else if (type == 1){
					// データベースからの引き継ぎ
		    		double Fi = 1;
		    		Matrix C(H.getColumn(j));
		    		Matrix CT(C.transposed());

					Complex CNorm2 = CT.multiplied(C).scalar();
					if (abs(CNorm2) < 1.0e-6){
					   	CNorm2 = 1.0e-6;
					}
					Complex CNorm2i = 1.0 / CNorm2;

					F.setRow(j, Fi * CNorm2i * CT);
					P = Matrix::I(numMic) - C * CNorm2i * CT;
  					setSeparationMatricesFromDatabase(sourceIndex, j, i, G);
				}
				else {
					// 初期値を入力
		    		double Fi = 1;
		    		Matrix C(H.getColumn(j));
		    		Matrix CT(C.transposed());

					// 2012/05/25 changed by N.Tanaka
					//Complex CNorm2i = 1.0 / CT.multiplied(C).scalar();
					Complex CNorm2 = CT.multiplied(C).scalar();
					if (abs(CNorm2) < 1.0e-6){
					   	CNorm2 = 1.0e-6;
					}
					Complex CNorm2i = 1.0 / CNorm2;

					F.setRow(j, Fi * CNorm2i * CT);
					P = Matrix::I(numMic) - C * CNorm2i * CT;
					G.setRow(j, Fi * CNorm2i * CT);
				}*/
				if (type == 0) { // 直近フレームにある
					// 直近フレームのデータからの引き継ぎ
					F.setRow(j, this->F[i].getRow(sourceIndex));
					P = this->P[i * this->sourcePositions.size() + sourceIndex];
					G.setRow(j, this->separationMatrices[i].getRow(sourceIndex));
				}
				else { // データベースにある, 見当たらない
					// Fs, Ps は初期値
		    		double Fi = 1;
		    		Matrix C(H.getColumn(j));
		    		Matrix CT(C.transposed());

					Complex CNorm2 = CT.multiplied(C).scalar();
					if (abs(CNorm2) < 1.0e-6) {
					   	CNorm2 = 1.0e-6;
					}
					Complex CNorm2i = 1.0 / CNorm2;

					F.setRow(j, Fi * CNorm2i * CT);
					P = Matrix::I(numMic) - C * CNorm2i * CT;

					if (type == 1) { // データベースにある
						// データベースからの引き継ぎ
						setSeparationMatricesFromDatabase(sourceIndex, j, i, G);
					}
					else { // (type == -1) 見当たらない
						// 初期値を入力
						G.setRow(j, Fi * CNorm2i * CT);
					}
				}

				newPs.push_back(P);
			}
	    }
		
		newGs.push_back(G);
		newFs.push_back(F);
	}

	/*== 分離行列データベースの座標を更新 ==*/
	updateSeparationMatricesDatabaseSource(sourcePositions, sourceIndicestypes, sourceIndices);
#if 0
int ifreqindex = 1;
std::cout << "size of newGS[" << ifreqindex << "] = (" << newGs[ifreqindex].getRowCount() << "," << newGs[ifreqindex].getColumnCount() << ")";
for (int k = 0 ; k < sourcePositions.size() ; k++){
std::cout << "BF->G(f" << ifreqindex << "s" << sourcePositions[k].id << "):";
for (int j = 0 ; j < this->micPositions.size() ; j++){
std::cout << newGs[ifreqindex].getElementAt(k, j) << ",";
}
std::cout << "@BF_LCMV::updateSources" << std::endl;
}
#endif

	/*== 結果の格納 ==*/
	this->sourcePositions = sourcePositions;
	newFs.swap(this->F);
	newPs.swap(this->P);
	newHs.swap(this->transformFunctions);
	newGs.swap(this->separationMatrices);
    }

    void
    LCMV::adaptSeparationMatrices()
    {
	int numSrc = this->sourcePositions.size();

	std::vector<Matrix> newGs;
//	for(int i = lowerfreq; i <= upperfreq ; i++){
	for(int i = 0; i < this->numFreq; i++){
	    const Matrix &H = this->transformFunctions[i];
	    int numMic = H.getRowCount();

	    Matrix G = this->separationMatrices[i];

	    // 対象周波数のみ分析	    
//    	if (i < lowerfreq || i > upperfreq){
	    if (i == 0 || i == this->numFreq-1 || i < lowerfreq || i > upperfreq) {
			newGs.push_back(G);
			continue;
	    }

	    for(int j = 0; j < numSrc; j++){
			Matrix        F  = this->F[i].getRow(j);
			const Matrix &P  = this->P[i * numSrc + j];
			Complex       y  = this->outputSpectrum.getElementAt(j, i);
			Matrix        x  = this->inputSpectrum.getColumn(i);

			double mu;
			if(this->muAdp){
			  // mu = norm(y) / (2.0 * (norm(y) * x.norm().real())) * this->mu; // original adaptive stepsize
				if(x.norm().real() < BF::BF_EPS){
					mu = 1.0 / (2.0 * x.norm().real() + BF::BF_EPS) * this->mu; // norm(y) is cancelled
				}else{
					mu = 1.0 / (2.0 * x.norm().real()) * this->mu; // norm(y) is cancelled
				}
			}else{
		  		mu = this->mu;
			}

#if ENABLE_JDEBUG
			double Jc = 1.0 / 2.0 * norm(y);
			if(j==0) std::cout << mu << " " << Jc << " ";		
#endif

			Matrix gn = (G.getRow(j) - mu * y * x.transposed()) * P + F;
			G.setRow(j, gn);
	    }

	    newGs.push_back(G);
	}

#if ENABLE_JDEBUG
	std::cout << std::endl;
#endif
	
	//fprintf(stdout, "mu=%f\n", mu);
	//this->separationMatrices[7].dump(stdout, "oldG[7]");
	//newGs[7].dump(stdout, "newG[7]");

	/*== 結果の格納 ==*/
	newGs.swap(this->separationMatrices);
    }
}
