// --------------------------------------------------------------------
#include "HarkFileIOByMatrix.h"

#include <vector>
#include <string.h>
#include <iostream>
// --------------------------------------------------------------------

#ifdef ENABLE_HARKIO3
char HARKFILEIOBYMAT_SIGNATURE_TFMAT[] = "transfer function";
char HARKFILEIOBYMAT_SIGNATURE_SPMAT[] = "separation matrix";
char HARKFILEIOBYMAT_STRTYPE_COMPLEX[] = "complex";
#endif

// --------------------------------------------------------------------
CHarkFileIOByMatrix::CHarkFileIOByMatrix()
{
	m_pcTFfilename = NULL;	///< TFt@C
	m_pcFileType = NULL;
#ifdef ENABLE_HARKIO3
	m_ptf = NULL;	///< TFt@C|C^
#else
#endif
}
// --------------------------------------------------------------------
CHarkFileIOByMatrix::~CHarkFileIOByMatrix()
{
	CloseTFfile();
}
// --------------------------------------------------------------------
// tft@C̃I[v(cFilenameɊJĂꍇ̓X[)
bool CHarkFileIOByMatrix::OpenTFfile(const char* cFilename, const char* cFileType)
{
	if (m_pcTFfilename){
		if (strcmp(m_pcTFfilename, cFilename) == 0){
			// łɊJĂ̂ŉȂ
			if (strcmp(m_pcFileType, cFileType) != 0){
				return false;	// Ǎ݃t@C^CvقȂ
			}
			return true;
		}
	}
	
	return (ReopenTFfile(cFilename, cFileType));
}
// --------------------------------------------------------------------
// tft@C̍ăI[v
bool CHarkFileIOByMatrix::ReopenTFfile(const char* cFilename, const char* cFileType)
{
	int i;
	
	if (m_pcTFfilename){
		CloseTFfile();
	}

#ifdef ENABLE_HARKIO3
#else
	if (strcmp(m_pcFileType, HARKFILEIOBYMAT_SIGNATURE_TFMAT) != 0 &&
		strcmp(m_pcFileType, HARKFILEIOBYMAT_SIGNATURE_SPMAT) != 0){
		return false;
	}
#endif
	
	m_pcTFfilename = (char *)malloc(strlen(cFilename)+1);
	strcpy(m_pcTFfilename, cFilename);
	m_pcFileType = (char *)malloc(strlen(cFileType)+1);
	strcpy(m_pcFileType, cFileType);
	
#ifdef ENABLE_HARKIO3
	m_ptf = harkio_TransferFunction_fromFile(m_pcTFfilename, m_pcFileType);
	if (!m_ptf){
		std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReopenTFfile By" << m_pcFileType << std::endl;
		free(m_pcTFfilename);
		free(m_pcFileType);
		m_pcTFfilename = NULL;
		m_pcFileType = NULL;
		return false;
	}
#else
	if (!m_ctf.OpenTFfile(m_pcTFfilename)){
		std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReopenTFfile By" << m_pcFileType << std::endl;
		free(m_pcTFfilename);
		free(m_pcFileType);
		m_pcTFfilename = NULL;
		m_pcFileType = NULL;
		return false;
	}
	if (strcmp(m_pcFileType, HARKFILEIOBYMAT_SIGNATURE_TFMAT) == 0){
		if (m_ctf.m_iFileType != FORMAT_TYPE_TF){
			std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReopenTFfile By" << m_pcFileType << "(Format type error)" << std::endl;
			free(m_pcTFfilename);
			free(m_pcFileType);
			m_pcTFfilename = NULL;
			m_pcFileType = NULL;
			m_ctf.CloseTFfile();
			return false;
		}
	}
	else if (strcmp(m_pcFileType, HARKFILEIOBYMAT_SIGNATURE_SPMAT) == 0){
		if (m_ctf.m_iFileType != FORMAT_TYPE_W){
			std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReopenTFfile By" << m_pcFileType << "(Format type error)" << std::endl;
			free(m_pcTFfilename);
			free(m_pcFileType);
			m_pcTFfilename = NULL;
			m_pcFileType = NULL;
			m_ctf.CloseTFfile();
			return false;
		}
	}
#endif
	
	return true;
}
// --------------------------------------------------------------------
// tft@C̃N[Y
bool CHarkFileIOByMatrix::CloseTFfile()
{
	if (m_pcTFfilename){
		free(m_pcTFfilename);
	}
	if (m_pcFileType){
		free(m_pcFileType);
	}
#ifdef ENABLE_HARKIO3
	if (m_ptf){
		harkio_TransferFunction_delete(&m_ptf);
	}	
#else
	m_ctf.CloseTFfile;
#endif

	// 
	m_pcTFfilename = NULL;
	m_pcFileType = NULL;
#ifdef ENABLE_HARKIO3
	m_ptf = NULL;
#else
#endif
}
// --------------------------------------------------------------------
// t@Cǂݍ
bool CHarkFileIOByMatrix::ReadHeader(const char* cFilename, const char* cFileType,
				long *plFs,
				long *plNFFT,
				long *plNSrc,
				long *plNMic)
{
	try {
		int i;

		if (!OpenTFfile(cFilename, cFileType)){
			return false;
		}
		
#ifdef ENABLE_HARKIO3
		harkio_TransferFunction* tf = m_ptf;
		
		if (plFs){ *plFs = tf->cfg->samplingRate; }
		if (plNFFT){ *plNFFT = tf->cfg->nfft; }
		if (plNSrc){ *plNSrc = tf->poses->size; }
		if (plNMic){ *plNMic = tf->mics->size; }
#else
		MICARY_TFDBFileHeader sHeader;
		m_ctf.ReadHeader(cFilename, &sHeader);
		if (plFs){ *plFs = sHeader.lFs; }
		if (plNFFT){ *plNFFT = sHeader.lNFFT; }
		if (plNSrc){ *plNSrc = sHeader.lNSrc; }
		if (plNMic){ *plNMic = sHeader.lNMic; }
#endif
		
	}
	catch(std::exception &ex){
	
		return false ;
	
	}

	return true ;
}
// --------------------------------------------------------------------
// TFf[^Ǎ
bool CHarkFileIOByMatrix::ReadTFData(const char* cFilename, 
				float thresh,
				BF::TransformFunctionDatabase &sDatabase)
{
	if (!OpenTFfile(cFilename, HARKFILEIOBYMAT_SIGNATURE_TFMAT)){
		return false;
	}
	
#ifdef ENABLE_HARKIO3
	try {
		harkio_TransferFunction* tf = m_ptf;
		int i, j, k;
		float fcoord[3];
		float *pfc;
		COMPLEX_TYPE value;
		BF::Complex	inputvalue;

		// Mic̎擾
		sDatabase.micLocations.clear();
		for(i = 0; i < tf->mics->size ; i++){
			if (tf->mics->pos[i]->sys != Cartesian){
				harkio_Position_Convert(tf->mics->pos[i], fcoord, Cartesian);
				pfc = fcoord;
			}
			else {
				pfc = tf->mics->pos[i]->coord;
			}
			BF::MicLocation mictmp(pfc[0], pfc[1], pfc[2], tf->mics->pos[i]->id);
			sDatabase.micLocations.push_back(mictmp);
		}
		
		// Src̎擾
		sDatabase.sourceLocations.clear();
		for(i = 0; i < tf->poses->size ; i++){
			if (tf->poses->pos[i]->sys != Cartesian){
				harkio_Position_Convert(tf->poses->pos[i], fcoord, Cartesian);
				pfc = fcoord;
			}
			else {
				pfc = tf->poses->pos[i]->coord;
			}
			BF::SourceLocation srctmp(pfc[0], pfc[1], pfc[2], tf->poses->pos[i]->id, tf->poses->pos[i]->id, true);
			sDatabase.sourceLocations.push_back(srctmp);
		}
		
		// TFf[^̎擾
		// size_is(numFreq), elemnent_is(Matrix(numMic, numSrc)) 
		sDatabase.Data.clear();
		for (k = 0 ; k < tf->cfg->nfft/2+1 ; k++){
			BF::Matrix m = BF::Matrix::O(tf->mics->size, tf->poses->size);
			for (j = 0 ; j < tf->poses->size ; j++){
				for (i = 0 ; i < tf->mics->size ; i++){
					harkio_Matrix_getValueComplex(tf->septfs[j], i, k, &value);
					inputvalue.real() = crealf(value);	// H=conj(TFCONJ)
					inputvalue.imag() = -cimagf(value);
					m.setElementAt(i, j, inputvalue);
				}
			}
			sDatabase.Data.push_back(m);
		}
		
		// Neighbors擾
		_CalcNeighbors(sDatabase.sourceLocations, thresh, sDatabase.sourceNeighbors);
	}
	catch(std::exception &ex){
		return false ;
	}
#else
    /* HARK `̃f[^x[Xǂݍ */
    MICARY_TFDBFileHeader tfdbh;
    CHarkFileIO harkFileIO;
    if(!harkFileIO.ReadHeader(this->tfFileName.c_str(), &tfdbh)){
		std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReadTFData By" << m_pcFileType << "(ReadHeader error)" << std::endl;
		return false;
	}
	

    MICARY_TFDatabase tfdb;
    TFDatabaseAlloc(&tfdb, tfdbh.lNFreq, tfdbh.lNSrc, tfdbh.lNMic);
    if(!harkFileIO.ReadInfo(this->tfFileName.c_str(), &tfdbh, &tfdb.micary_locaSource, &tfdb.micary_locaMic, &tfdb.micary_iaFreqLine)
       || !harkFileIO.ReadData(this->tfFileName.c_str(), &tfdb.micary_cmpaTF)){
		std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReadTFData By" << m_pcFileType << "(ReadInfo/Data error)" << std::endl;
		return false;
    }

    /* f[^x[X`ϊ */
    int numSrc  = tfdb.micary_locaSource.ha_numLocation;
    int numMic  = tfdb.micary_locaMic.ha_numLocation;
    int numFreq = tfdb.micary_iaFreqLine.ha_numValue;
    sDatabase.sourceLocations.clear();
    for(int i = 0; i < numSrc; i++){
		MICARY_Location &l = tfdb.micary_locaSource.micary_Location[i];
		BF::SourceLocation sl(l.ha_fX, l.ha_fY, l.ha_fZ, tfdb.micary_locaSource.ha_idLocation[i], 
							  tfdb.micary_locaSource.ha_idLocation[i], true);
		sDatabase.sourceLocations.push_back(sl);
    }
    sDatabase.micLocations.clear();
    for(int i = 0; i < numMic; i++){
		MICARY_Location &l = tfdb.micary_locaMic.micary_Location[i];
		BF::MicLocation ml(l.ha_fX, l.ha_fY, l.ha_fZ, tfdb.micary_locaMic.ha_idLocation[i]);
		sDatabase.micLocations.push_back(ml);
    }
    sDatabase.Data.clear();
    for(int i = 0; i < numFreq; i++){
		BF::Matrix tf(numMic, numSrc);

		for(int j = 0; j < numMic; j++){
		    for(int k = 0; k < numSrc; k++){
				// micary_cmpaData->ha_cmpValue(g((}CN)))̏ɓ
				HA_Complex &v =tfdb.micary_cmpaTF.ha_cmpValue[(i * numSrc + k) * numMic + j];
				// H=conj(TFCONJ)
				tf.setElementAt(j, k, v.re, -v.im);
		    }
		}
		sDatabase.Data.push_back(tf);
    }
	TFDatabaseFree(&tfdb);

	// Neighbors擾
	_CalcNeighbors(sDatabase.sourceLocations, thresh, sDatabase.sourceNeighbors);
#endif
	return true;
}
// --------------------------------------------------------------------
// Wf[^Ǎ
bool CHarkFileIOByMatrix::ReadWData(const char* cFilename, 
				BF::Database &sDatabase)
{
	if (!OpenTFfile(cFilename, HARKFILEIOBYMAT_SIGNATURE_SPMAT)){
		return false;
	}
	
#ifdef ENABLE_HARKIO3
	try {
		harkio_TransferFunction* tf = m_ptf;
		int i, j, k;
		float fcoord[3];
		float *pfc;
		COMPLEX_TYPE value;
		BF::Complex	inputvalue;
		// Mic̎擾
		sDatabase.micLocations.clear();
		for(i = 0; i < tf->mics->size ; i++){
			if (tf->mics->pos[i]->sys != Cartesian){
				harkio_Position_Convert(tf->mics->pos[i], fcoord, Cartesian);
				pfc = fcoord;
			}
			else {
				pfc = tf->mics->pos[i]->coord;
			}
			BF::MicLocation mictmp(pfc[0], pfc[1], pfc[2], tf->mics->pos[i]->id);
			sDatabase.micLocations.push_back(mictmp);
		}
		
		// Src̎擾
		sDatabase.sourceLocations.clear();
		for(i = 0; i < tf->poses->size ; i++){
			if (tf->poses->pos[i]->sys != Cartesian){
				harkio_Position_Convert(tf->poses->pos[i], fcoord, Cartesian);
				pfc = fcoord;
			}
			else {
				pfc = tf->poses->pos[i]->coord;
			}
			BF::SourceLocation srctmp(pfc[0], pfc[1], pfc[2], tf->poses->pos[i]->id, tf->poses->pos[i]->id, true);
			sDatabase.sourceLocations.push_back(srctmp);
		}
		
		// Wf[^̎擾
		// size_is(numSrc), elemnent_is(Matrix(numMic, numFreq))
		sDatabase.Data.clear();
		for (j = 0 ; j < tf->poses->size ; j++){
			BF::Matrix m = BF::Matrix::O(tf->mics->size, tf->cfg->nfft/2+1);
			for (k = 0 ; k < tf->cfg->nfft/2+1 ; k++){
				for (i = 0 ; i < tf->mics->size ; i++){
					harkio_Matrix_getValueComplex(tf->septfs[j], i, k, &value);
					inputvalue.real() = crealf(value);
					inputvalue.imag() = cimagf(value);
					m.setElementAt(i, k, inputvalue);
				}
			}
			sDatabase.Data.push_back(m);
		}	
	}
	catch(std::exception &ex){
		return false ;
	}
#else
    /* HARK `̃f[^x[Xǂݍ */
    MICARY_TFDBFileHeader tfdbh;
    CHarkFileIO harkFileIO;
    if(!harkFileIO.ReadHeader(this->tfFileName.c_str(), &tfdbh)){
		std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReadTFData By" << m_pcFileType << "(ReadHeader error)" << std::endl;
		return false;
	}
	
    MICARY_TFDatabase tfdb;
    TFDatabaseAlloc(&tfdb, tfdbh.lNFreq, tfdbh.lNSrc, tfdbh.lNMic);
    if(!harkFileIO.ReadInfo(this->tfFileName.c_str(), &tfdbh, &tfdb.micary_locaSource, &tfdb.micary_locaMic, &tfdb.micary_iaFreqLine)
       || !harkFileIO.ReadData(this->tfFileName.c_str(), &tfdb.micary_cmpaTF)){
		std::cerr << "Failed to open " << m_pcTFfilename << "@CHarkFileIOByMatrix::ReadTFData By" << m_pcFileType << "(ReadInfo/Data error)" << std::endl;
		return false;
    }

    /* f[^x[X`ϊ */
    int numSrc  = tfdb.micary_locaSource.ha_numLocation;
    int numMic  = tfdb.micary_locaMic.ha_numLocation;
    int numFreq = tfdb.micary_iaFreqLine.ha_numValue;
    sDatabase.sourceLocations.clear();
    for(int i = 0; i < numSrc; i++){
		MICARY_Location &l = tfdb.micary_locaSource.micary_Location[i];
		BF::SourceLocation sl(l.ha_fX, l.ha_fY, l.ha_fZ, tfdb.micary_locaSource.ha_idLocation[i], 
							  tfdb.micary_locaSource.ha_idLocation[i], true);
		sDatabase.sourceLocations.push_back(sl);
    }
    sDatabase.micLocations.clear();
    for(int i = 0; i < numMic; i++){
		MICARY_Location &l = tfdb.micary_locaMic.micary_Location[i];
		BF::MicLocation ml(l.ha_fX, l.ha_fY, l.ha_fZ, tfdb.micary_locaMic.ha_idLocation[i]);
		sDatabase.micLocations.push_back(ml);
    }
    sDatabase.Data.clear();
    // size_is(numSrc), elemnent_is(Matrix(numMic, numFreq))
    for(int k = 0; k < numSrc; k++){
		BF::Matrix tf = Matrix::O(numMic, numFreq);
	    for(int i = 0; i < numFreq; i++){
			for(int j = 0; j < numMic; j++){
				// micary_cmpaData->ha_cmpValue(g(}CN()))̏ɓ
				HA_Complex &v =tfdb.micary_cmpaTF.ha_cmpValue[(i * numMic + j) * numSrc + k];
				tf.setElementAt(k, j, v.re, v.im);
		    }
		}
		sDatabase.Data.push_back(tf);
    }
	TFDatabaseFree(&tfdb);
#endif
	return true;
}
// --------------------------------------------------------------------
// NeighborsXg擾
void CHarkFileIOByMatrix::_CalcNeighbors(std::vector<BF::SourceLocation> sourceLocations, float thresh, std::vector<std::vector<int> > &viNbrs)
{
	std::vector<std::pair<float, int> > vifclosestIDs;
	int i, j;

	viNbrs.clear();
	viNbrs.resize(sourceLocations.size());
	
	for (i = 0 ; i < sourceLocations.size() ; i++){
		// 
		vifclosestIDs.clear();
		
		// threshouldȓ
		for(j = 0 ; j < sourceLocations.size() ; j++)
		{
			float tmpdist = sourceLocations[i].euclidDistance(sourceLocations[j]);
			if (tmpdist < thresh){
				vifclosestIDs.push_back(std::pair<float, int>(tmpdist, sourceLocations[j].id));
			}
   	    }
		
		// sort
		sort(vifclosestIDs.begin(), vifclosestIDs.end());

		// Neighbor\̓ɕۑ
		viNbrs[i].resize(vifclosestIDs.size());
		for (j = 0 ; j < vifclosestIDs.size() ; j++){
			viNbrs[i][j] = vifclosestIDs[j].second;
		}
		
	}
	
}
// --------------------------------------------------------------------
// f[^
bool CHarkFileIOByMatrix::WriteWData(const char* cFilename, 
				long lFs,
				long lNFFT,
				BF::Database sDatabase)
{
#ifdef ENABLE_HARKIO3
	try{
		int i, j, k;
		float  coord[3];
		COMPLEX_TYPE inputvalue;

		// WƂĕۑ
		harkio_TransferFunction *tf = harkio_TransferFunction_new();
		if (!tf){
			std::cerr << "Failed to create harkio_Transferfunction @CHarkFileIOByMatrix::WriteWData" << std::endl;
			return false;
		}

		// config
		tf->cfg->samplingRate = lFs;
		tf->cfg->nfft = lNFFT;

		harkio_Position *pos;

		// ʒu̕ۑ
	        tf->poses->type = (char*) malloc(sizeof(char) * (strlen("soundsource") + 1));
		strcpy(tf->poses->type, "soundsource");
		for (i = 0 ; i < sDatabase.sourceLocations.size() ; i++){
			coord[0] = sDatabase.sourceLocations[i].x;
			coord[1] = sDatabase.sourceLocations[i].y;
			coord[2] = sDatabase.sourceLocations[i].z;
			pos = harkio_Position_new(sDatabase.sourceLocations[i].id, Cartesian, coord, NULL);
			harkio_Positions_append(tf->poses, pos);
		}

		// Neighbors̐ݒ
		if (tf->nbrs){
			harkio_Neighbors_delete(&tf->nbrs);
			tf->nbrs = NULL;
		}
		tf->nbrs = harkio_Neighbors_new(tf->poses);
		harkio_Neighbors_calcNeighbors(tf->nbrs, NearestNeighbor, 1, 300.0);

		// }CNʒu̕ۑ
	        tf->mics->type = (char*) malloc(sizeof(char) * (strlen("microphone") + 1));
		strcpy(tf->mics->type, "microphone");
		for (i = 0 ; i < sDatabase.micLocations.size() ; i++){
			coord[0] = sDatabase.micLocations[i].x;
			coord[1] = sDatabase.micLocations[i].y;
			coord[2] = sDatabase.micLocations[i].z;
			pos = harkio_Position_new(sDatabase.micLocations[i].id, Cartesian, coord, NULL);
			harkio_Positions_append(tf->mics, pos);
		}


		// septfsɕۑ
		tf->loctfs = NULL;
		tf->septfs = (harkio_Matrix **) malloc(sizeof(harkio_Matrix *) * tf->poses->size);
		for (i = 0 ; i < tf->poses->size ; i++){
			tf->septfs[i] = harkio_Matrix_new(HARKFILEIOBYMAT_STRTYPE_COMPLEX, tf->mics->size, lNFFT/2+1);
		}
		// sDatabase.Data: size_is(numSrc), elemnent_is(Matrix(numMic, numFreq))
		for (k=0 ; k < lNFFT/2+1 ; k++){
			for (i = 0 ; i < tf->mics->size ; i++){
				for (j = 0 ; j < tf->poses->size ; j++){
					BF::Complex cvalue = sDatabase.Data[j].getElementAt(i, k);
					inputvalue =  cvalue.real() + cvalue.imag()*_Complex_I;
					harkio_Matrix_setValueComplex(tf->septfs[j], i, k, inputvalue);
				}
			}
		}

		char *cName = (char*)malloc((strlen(cFilename)+1)*sizeof(char));
		strcpy(cName, cFilename);
		harkio_TransferFunction_write(tf, cName, HARKFILEIOBYMAT_SIGNATURE_SPMAT);
		free(cName);
		
		harkio_TransferFunction_delete(&tf);
	}
	catch(std::exception &ex){
	
		return false ;
	
	}
#else
	int numMic = sDatabase.micLocations.size();
	int numSrc = sDatabase.sourceLocations.size();
	int numFreq = lNFFT/2+1;

	MICARY_SMDatabase smDatabase = {0};
		
	SMDatabaseAlloc(&smDatabase, 0, 0, 0);
	SMDatabaseAlloc(&smDatabase, numFreq, numSrc, numMic);

	for(int i = 0; i < numMic; i++){
	    const BF::MicLocation &ml = sDatabase.micLocations[i];
	    smDatabase.micary_locaMic.ha_idLocation[i] = ml.id;
	    smDatabase.micary_locaMic.micary_Location[i].ha_fX = ml.x;
	    smDatabase.micary_locaMic.micary_Location[i].ha_fY = ml.y;
	    smDatabase.micary_locaMic.micary_Location[i].ha_fZ = ml.z;
	    smDatabase.micary_locaMic.micary_Location[i].ha_idPos = ml.id;
	}

	for(int i = 0; i < numSrc; i++){
	    const BF::SourceLocation &sl = sDatabase.sourceLocations[i];
	    smDatabase.micary_locaSource.ha_idLocation[i] = sl.sourceID;
	    smDatabase.micary_locaSource.micary_Location[i].ha_fX = sl.x;
	    smDatabase.micary_locaSource.micary_Location[i].ha_fY = sl.y;
	    smDatabase.micary_locaSource.micary_Location[i].ha_fZ = sl.z;
	    smDatabase.micary_locaSource.micary_Location[i].ha_idPos = sl.id;
	}

	for(int i = 0 ; i < numFreq; i++){
	    smDatabase.micary_smSeparation.micary_iaFreqLinex.ha_iValue[i] = i;
	}

	int c = 0;
	// sDatabase.Data: size_is(numSrc), elemnent_is(Matrix(numMic, numFreq))
    for(int j = 0 ; j < numSrc; j++){
	    const BF::Matrix &mat = sDatabase.Data[i];
		for(int i = 0 ; i < numFreq; i++){
			for(int k = 0 ; k < numMic; k++){
				// micary_cmpaData->ha_cmpValue(g(}CN()))̏ɓ
				c = (i * numMic + j) * numSrc + k;
		    	smDatabase.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[c].re = mat.getElementAt(j, k).real();
			    smDatabase.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[c].im = mat.getElementAt(j, k).imag();
			}
	    }
	}

	MICARY_TFDBFileHeader tfdbfh;
	tfdbfh.lFs       = lFs;
	tfdbfh.lNFFT     = lNFFT;
	tfdbfh.lNMic     = numMic;
	tfdbfh.lNSrc     = numSrc;
	tfdbfh.lNFreq    = numFreq;
	tfdbfh.iVersion  = 1;
	tfdbfh.iFileType = FORMAT_TYPE_W;

	CHarkFileIO cHark;
	if(!cHark.WriteData(cFilename.c_str(), &tfdbfh,
			    smDatabase.micary_smSeparation.micary_cmpaSeparation,
			    smDatabase.micary_smSeparation.micary_iaFreqLinex,
			    smDatabase.micary_locaSource,
			    smDatabase.micary_locaMic,
			    "",
			    "",
			    "",
			    "",
			    "",
			    "W_EXPORT")){
		std::cerr << "Failed to WriteData @CHarkFileIOByMatrix::WriteWData" << std::endl;
	    SMDatabaseFree(&smDatabase);
	    return false;
	}

	SMDatabaseFree(&smDatabase);
#endif

	return true;
}
// --------------------------------------------------------------------


