#include <string>
#include <iostream>
#include <cstdlib>
#include <complex>
#include <cmath>
#include <string.h>

#include "BufferedNode.h"
#include "Vector.h"
#include "Matrix.h"
#include "Map.h"
#include "Source.h"

using namespace std;
using namespace FD;

// Node name
class TestPublish;

DECLARE_NODE(TestPublish);
/*Node
 *
 * @name TestPublish
 * @category HARK:BeamForming
 * @description Test BeamForming Node
 *
 * @output_name OUTPUT_X
 * @output_type Matrix<complex<float> >
 * @output_description
 *
 * @output_name OUTPUT_Y
 * @output_type Matrix<complex<float> >
 * @output_description
 *
 * @output_name OUTPUT_G
 * @output_type Matrix<complex<float> >
 * @output_description
 *
 * @output_name OUTPUT_S
 * @output_type Vector<ObjectRef>
 * @output_description
 *
 * @output_name OUTPUT_N
 * @output_type Vector<ObjectRef>
 * @output_description
 *
 * @parameter_name FILE_X
 * @parameter_type string
 * @parameter_description
 *
 * @parameter_name FILE_Y
 * @parameter_type string
 * @parameter_description
 *
 * @parameter_name FILE_G
 * @parameter_type string
 * @parameter_description
 *
 END*/

class TestPublish : public BufferedNode
{
public:
    TestPublish(string nodeName, ParameterSet params)
	: BufferedNode(nodeName, params), fpX(NULL), fpY(NULL), fpG(NULL)
    {
	/*== FlowDesigner 入出力インタフェース ==*/

	/* 入出力IDの割り当て */
	this->outputX_ID = addOutput("OUTPUT_X");
	this->outputY_ID = addOutput("OUTPUT_Y");
	this->outputG_ID = addOutput("OUTPUT_G");
	this->outputS_ID = addOutput("OUTPUT_S");
	this->outputN_ID = addOutput("OUTPUT_N");

	this->xFileName = object_cast<string>(parameters.get("FILE_X"));
	this->yFileName = object_cast<string>(parameters.get("FILE_Y"));
	this->gFileName = object_cast<string>(parameters.get("FILE_G"));

	if((this->fpX = fopen(this->xFileName.c_str(), "r")) == NULL){
	    throw (new NodeException(NULL, "TestPublish: Failed to open X file.", __FILE__, __LINE__));
	}
	if((this->fpY = fopen(this->yFileName.c_str(), "r")) == NULL){
	    fclose(this->fpX);
	    throw (new NodeException(NULL, "TestPublish: Failed to open Y file.", __FILE__, __LINE__));
	}
	if((this->fpG = fopen(this->gFileName.c_str(), "r")) == NULL){
	    fclose(this->fpX);
	    fclose(this->fpY);
	    throw (new NodeException(NULL, "TestPublish: Failed to open G file.", __FILE__, __LINE__));
	}

	char buffer[1024];
	while(fgets(buffer, 1024, fpX)){
	    int n = 0, ff;
	    char *p;
	    if((p = strchr(buffer, '#')) != NULL){
		*p = '\0';
	    }
	    if((n = sscanf(buffer, " %d %d %d %d", &this->dimXY, &this->dimXX, &this->countX, &ff)) == 4){
		this->countX *= ff;
		break;
	    }else if(n == 3){
		break;
	    }else{
//		fprintf(stderr, "??> %s\n", buffer);
//		throw (new NodeException(NULL, "TestPublish: Failed to read from X file.", __FILE__, __LINE__));
	    }
	}
	while(fgets(buffer, 1024, fpY)){
	    int n = 0, ff;
	    char *p;
	    if((p = strchr(buffer, '#')) != NULL){
		*p = '\0';
	    }
	    if((n = sscanf(buffer, " %d %d %d %d", &this->dimYY, &this->dimYX, &this->countY, &ff)) == 4){
		this->countY *= ff;
		break;
	    }else if(n == 3){
		break;
	    }else{
//		fprintf(stderr, "??> %s\n", buffer);
//		throw (new NodeException(NULL, "TestPublish: Failed to read from Y file.", __FILE__, __LINE__));
	    }
	}
	while(fgets(buffer, 1024, fpG)){
	    int n = 0, ff;
	    char *p;
	    if((p = strchr(buffer, '#')) != NULL){
		*p = '\0';
	    }
	    if((n = sscanf(buffer, " %d %d %d %d", &this->dimGY, &this->dimGX, &this->countG, &ff)) == 4){
		this->countG *= ff;
		break;
	    }else if(n == 3){
		break;
	    }else{
//		fprintf(stderr, "??> %s\n", buffer);
//		throw (new NodeException(NULL, "TestPublish: Failed to read from G file.", __FILE__, __LINE__));
	    }
	}
    }

    virtual ~TestPublish()
    {
	if(this->fpX){
	    fclose(this->fpX);
	}
	if(this->fpY){
	    fclose(this->fpY);
	}
	if(this->fpG){
	    fclose(this->fpG);
	}
    }

public:
    void calculate(int output_id, int count, Buffer &out)
    {
	/*== 入出力の準備 ==*/

	/* 出力スペクトルの領域を準備 */
	ObjectRef outputXTemp = ObjectRef(new Matrix<complex<float> >(this->dimXY, this->dimXX));
	ObjectRef outputYTemp = ObjectRef(new Matrix<complex<float> >(this->dimYY, this->dimYX));
        ObjectRef outputGTemp = ObjectRef(new Matrix<complex<float> >());
        ObjectRef outputSTemp = ObjectRef(new Vector<ObjectRef>());
        ObjectRef outputNTemp = ObjectRef(new Vector<ObjectRef>());

        Matrix<complex<float> > &outputXValue = object_cast<Matrix<complex<float > > >(outputXTemp);
	Matrix<complex<float> > &outputYValue = object_cast<Matrix<complex<float > > >(outputYTemp);
	Matrix<complex<float> > &outputGValue = object_cast<Matrix<complex<float > > >(outputGTemp);
	Vector<ObjectRef>       &outputSValue = object_cast<Vector<ObjectRef> >(outputSTemp);
	Vector<ObjectRef>       &outputNValue = object_cast<Vector<ObjectRef> >(outputNTemp);

	/*== 出力の準備 ==*/
	char buffer[1024];
	for(int k = 0; k < this->dimXX; k++){
	    for(int j = 0; j < this->dimXY; j++){
		double re, im;
		fgets(buffer, 1024, this->fpX);
		sscanf(buffer, " (%lf,%lf)", &re, &im);
		outputXValue(j, k) = complex<float>(re, im);
	    }
	}
//        cout << "X: " << outputXValue << endl;

	for(int k = 0; k < this->dimYX; k++){
	    for(int j = 0; j < this->dimYY; j++){
		double re, im;
		fgets(buffer, 1024, this->fpY);
		sscanf(buffer, " (%lf,%lf)", &re, &im);
		outputYValue(j, k) = complex<float>(re, im);
	    }
	}
//        cout << "Y: " << outputYValue << endl;

//        cout << "G: " << endl;
	for(int i = 0; i < this->dimYX; i++){
	    Matrix<complex<float> > g(this->dimYY, this->dimXX);
	    for(int k = 0; k < this->dimXX; k++){
		for(int j = 0; j < this->dimYY; j++){
		    double re, im;
		    fgets(buffer, 1024, this->fpG);
		    sscanf(buffer, " (%lf,%lf)", &re, &im);
		    g(j, k) = complex<float>(re, im);
		}
	    }
//            cout << g << endl;
	}

        for(int k = 0; k < 2; k++){
	    RCPtr<Source> s(new Source());

	    double r  = 1.5;
	    double th = k * 60.0 / 180.0 * M_PI;
	    s->x[0] = r * cos(th);
	    s->x[1] = r * sin(th);
	    s->x[2] = 0;
	    s->id   = k;
	    s->remaining = 0;

//	    fprintf(stdout, "count=%d\n", count);
	    if(k == 1 && ((count + 0) / 100) % 2 > 0){
		outputNValue.push_back(s);
	    }else{
		outputSValue.push_back(s);
	    }
	}

	/*== 処理結果を出力スペクトルの領域へ格納 ==*/
	(*(outputs[this->outputX_ID].buffer))[count] = outputXTemp;
	(*(outputs[this->outputY_ID].buffer))[count] = outputYTemp;
	(*(outputs[this->outputG_ID].buffer))[count] = outputGTemp;
	(*(outputs[this->outputS_ID].buffer))[count] = outputSTemp;
	(*(outputs[this->outputN_ID].buffer))[count] = outputNTemp;
    }

private:
    int outputX_ID, outputY_ID, outputG_ID, outputS_ID, outputN_ID;
    string xFileName, yFileName, gFileName;
    int dimXX, dimXY, dimYX, dimYY, dimGX, dimGY;
    int countX, countY, countG;

    FILE *fpX, *fpY, *fpG, *fpS;
};
